"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabasesNavigationTree = exports.NavigationWithPlaceholder = void 0;
const react_1 = __importStar(require("react"));
const react_window_1 = require("react-window");
const react_virtualized_auto_sizer_1 = __importDefault(require("react-virtualized-auto-sizer"));
const compass_components_1 = require("@mongodb-js/compass-components");
const placeholder_item_1 = require("./placeholder-item");
const constants_1 = require("./constants");
const database_item_1 = require("./database-item");
const collection_item_1 = require("./collection-item");
const use_virtual_navigation_tree_1 = require("./use-virtual-navigation-tree");
const databases_placeholder_1 = require("./databases-placeholder");
const collectionItemContainer = (0, compass_components_1.css)({
    position: 'relative',
});
const NavigationItem = (0, react_1.memo)(function NavigationItem({ index, style, data }) {
    const { items, isReadOnly, activeNamespace, currentTabbable, onDatabaseExpand, onNamespaceAction, } = data;
    const itemData = items[index];
    if (itemData.type === 'database') {
        return (react_1.default.createElement(database_item_1.DatabaseItem, { style: style, isReadOnly: isReadOnly, isActive: itemData.id === activeNamespace, isTabbable: itemData.id === currentTabbable, onNamespaceAction: onNamespaceAction, onDatabaseExpand: onDatabaseExpand, ...itemData }));
    }
    return (react_1.default.createElement("div", { className: collectionItemContainer },
        react_1.default.createElement(compass_components_1.FadeInPlaceholder, { isContentReady: itemData.type !== 'placeholder', contentContainerProps: { style }, fallbackContainerProps: { style }, content: () => {
                return (itemData.type !== 'placeholder' && (react_1.default.createElement(collection_item_1.CollectionItem, { isReadOnly: isReadOnly, isActive: itemData.id === activeNamespace, isTabbable: itemData.id === currentTabbable, onNamespaceAction: onNamespaceAction, ...itemData })));
            }, fallback: () => react_1.default.createElement(placeholder_item_1.PlaceholderItem, null) })));
}, react_window_1.areEqual);
const navigationTree = (0, compass_components_1.css)({
    flex: '1 0 auto',
});
const DatabasesNavigationTree = ({ databases, expanded = {}, activeNamespace = '', onDatabaseExpand, onNamespaceAction, isReadOnly = false, __TEST_REACT_AUTOSIZER_DEFAULT_WIDTH = null, __TEST_REACT_AUTOSIZER_DEFAULT_HEIGHT = null, __TEST_REACT_WINDOW_OVERSCAN = null, }) => {
    const listRef = (0, react_1.useRef)(null);
    const id = (0, compass_components_1.useId)();
    const items = (0, react_1.useMemo)(() => {
        return databases
            .map(({ _id: id, name, collections, collectionsLength, collectionsStatus }, dbIndex) => {
            const isExpanded = expanded[id];
            const database = {
                key: String(dbIndex),
                level: 1,
                id,
                name,
                type: 'database',
                isExpanded,
                setSize: databases.length,
                posInSet: dbIndex + 1,
            };
            if (!isExpanded) {
                return [database];
            }
            const areCollectionsReady = ['ready', 'refreshing', 'error'].includes(collectionsStatus);
            const placeholdersLength = Math.min(collectionsLength, constants_1.MAX_COLLECTION_PLACEHOLDER_ITEMS);
            return [database].concat(areCollectionsReady
                ? collections.map(({ _id: id, name, type }, index) => ({
                    key: `${dbIndex}-${index}`,
                    level: 2,
                    id,
                    name,
                    type: type,
                    setSize: collections.length,
                    posInSet: index + 1,
                }))
                : Array.from({ length: placeholdersLength }, (_, index) => ({
                    key: `${dbIndex}-${index}`,
                    type: 'placeholder',
                })));
        })
            .flat();
    }, [databases, expanded]);
    const onExpandedChange = (0, react_1.useCallback)((item, isExpanded) => {
        onDatabaseExpand(item.id, isExpanded);
    }, [onDatabaseExpand]);
    const onFocusMove = (0, react_1.useCallback)((item) => {
        const idx = items.indexOf(item);
        if (idx >= 0) {
            listRef.current?.scrollToItem(idx);
        }
    }, [items]);
    const [rootProps, currentTabbable] = (0, use_virtual_navigation_tree_1.useVirtualNavigationTree)({
        items: items,
        activeItemId: activeNamespace,
        onExpandedChange,
        onFocusMove,
    });
    const itemData = (0, react_1.useMemo)(() => {
        return {
            items,
            isReadOnly,
            activeNamespace,
            currentTabbable,
            onNamespaceAction,
            onDatabaseExpand,
        };
    }, [
        items,
        isReadOnly,
        activeNamespace,
        currentTabbable,
        onNamespaceAction,
        onDatabaseExpand,
    ]);
    const getItemKey = (0, react_1.useCallback)((index, data) => {
        return data.items[index].key;
    }, []);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(compass_components_1.VisuallyHidden, { id: id }, "Databases and Collections"),
        react_1.default.createElement("div", { role: "tree", "aria-labelledby": id, className: navigationTree, ...rootProps, "data-testid": "databases-and-collections" },
            react_1.default.createElement(react_virtualized_auto_sizer_1.default, { disableWidth: Boolean(__TEST_REACT_AUTOSIZER_DEFAULT_WIDTH), disableHeight: Boolean(__TEST_REACT_AUTOSIZER_DEFAULT_HEIGHT) }, ({ width = __TEST_REACT_AUTOSIZER_DEFAULT_WIDTH, height = __TEST_REACT_AUTOSIZER_DEFAULT_HEIGHT, }) => (react_1.default.createElement(react_window_1.FixedSizeList, { ref: listRef, width: width, height: height, itemData: itemData, itemCount: items.length, itemSize: constants_1.DATABASE_ROW_HEIGHT, itemKey: getItemKey, overscanCount: __TEST_REACT_WINDOW_OVERSCAN ?? 5 }, NavigationItem))))));
};
exports.DatabasesNavigationTree = DatabasesNavigationTree;
const container = (0, compass_components_1.css)({
    display: 'flex',
    flex: '1 0 auto',
});
const contentContainer = (0, compass_components_1.css)({
    display: 'flex',
    flex: '1 0 auto',
});
const NavigationWithPlaceholder = ({ isReady, ...props }) => {
    return (react_1.default.createElement(compass_components_1.FadeInPlaceholder, { className: container, contentContainerProps: { className: contentContainer }, isContentReady: isReady, content: () => {
            return react_1.default.createElement(DatabasesNavigationTree, { ...props });
        }, fallback: () => {
            return react_1.default.createElement(databases_placeholder_1.DatabasesPlaceholder, null);
        } }));
};
exports.NavigationWithPlaceholder = NavigationWithPlaceholder;
//# sourceMappingURL=databases-navigation-tree.js.map